// This code includes various functions for managing and interacting with a table of class schedules.
// It initializes variables and event handlers, handles AJAX requests, adds, deletes, sorts, filters, and prints records in the table.

var accId = new URLSearchParams(window.location.search).get('accId'); // Get accId from URL parameters
var upcomingCSArray;
var completedCSArray;
var csArray;
var table;

function getClassSchedules(accountId) {
    $("#loadingTable").show();
    retrieveMultiple(`${window.location.origin}/_api/bb_classschedules?$select=bb_name,statecode,bb_starttime,bb_endtime,bb_format,bb_type,createdon,_bb_bb_educatorlocationid_value,bb_sid,_bb_classid_value,bb_classscheduleid,_bb_accountid_value,modifiedon&$filter=(_bb_accountid_value eq ${accountId} and (statuscode eq 2 or (Microsoft.Dynamics.CRM.OlderThanXMinutes(PropertyName='bb_enddate',PropertyValue=1) and Microsoft.Dynamics.CRM.LastXYears(PropertyName='bb_enddate',PropertyValue=2))))&$orderby=bb_starttime desc`)
        .then((data1) => {
            completedCSArray = data1.value;
            retrieveMultiple(`${window.location.origin}/_api/bb_classschedules?$select=bb_name,statecode,bb_starttime,bb_endtime,bb_format,bb_type,createdon,_bb_bb_educatorlocationid_value,bb_sid,_bb_classid_value,bb_classscheduleid,_bb_accountid_value,modifiedon&$filter=(_bb_accountid_value eq ${accountId} and Microsoft.Dynamics.CRM.NextXYears(PropertyName='bb_enddate',PropertyValue=5) and _bb_hybridparent_value eq null)&$orderby=bb_starttime asc`)
                .then((data2) => {
                    upcomingCSArray = data2.value;
                    generateCSList();
                })
        });
}

// Function to generate a list of class schedules and populate the table
function generateCSList() {
    csArray = upcomingCSArray;

    // Initialize the table with specified columns and sort handler
    table = new webAPIExampleTable({
        columns: [{
            name: '_bb_classid_value@OData.Community.Display.V1.FormattedValue',
            label: 'Class',
        }, {
            name: 'bb_sid',
            label: 'Id',
        }, {
            name: 'bb_starttime@OData.Community.Display.V1.FormattedValue',
            label: 'Start',
        }, {
            name: 'bb_endtime@OData.Community.Display.V1.FormattedValue',
            label: 'End',
        }, {
            name: '_bb_bb_educatorlocationid_value@OData.Community.Display.V1.FormattedValue',
            label: 'Location',
        }, {
            name: 'bb_format@OData.Community.Display.V1.FormattedValue',
            label: 'Format',
        }, {
            name: 'bb_type@OData.Community.Display.V1.FormattedValue',
            label: 'Type',
        }, {
            name: 'createdon@OData.Community.Display.V1.FormattedValue',
            label: 'Created',
        }],
        data: [],
        sortHandler: sortTable
    });

    // Populate the table with data if csArray is not null
    setCSTableData(undefined, "class-schedules");
}

function setCSTableData(filterdData, parentUrl) {
    if (csArray != null && csArray.length > 0) {
        $("#loadingTable").hide();
        $("#emptymessage").hide();
        $("#dataTable").show();
        let dataArray = csArray;

        if (filterdData !== undefined) {
            dataArray = filterdData;
        }

        table.setData(_.map(dataArray, function (record) {
            record.partialUrl = record.statecode == 1 || record.statuscode == 121120001 ? "classschedule-ReadOnly" : "class-schedule-edit";
            record.parentUrl = parentUrl;
            record.id = record.bb_classscheduleid;
            let baseUrl = `${window.location.origin}/${parentUrl}/${record.partialUrl}/?refid=${record.bb_classscheduleid}&accId=${accId}`;
            if (record.statecode != 1 && record.statuscode != 121120001) {
                record.actions = `
                    <td>
                        <a href="${baseUrl}" class="btn btn-default" style="min-width: 100px;">
                            <i class="glyphicon glyphicon-pencil" aria-hidden="true"></i> Edit
                        </a>
                    </td>`;
            } else {
                record.actions = `
                    <td>
                        <a href="${baseUrl}" class="btn btn-default" style="min-width: 100px;">
                            <i class="glyphicon glyphicon-eye-open" aria-hidden="true"></i> View
                        </a>
                    </td>`;
            }
            return record;
        }));
    } else {
        $("#loadingTable").hide();
        $("#dataTable").hide();
        $("#emptymessage").show();
    }

    $('#searchInput').on('input', function () {
        var query = $(this).val();
        filterTable(query);
    });

    $('#printButton').on('click', function () {
        printTable();
    });

    $('.delete').on('click', function (e) {
        e.preventDefault();
        dataObj = getDataObject($(this).closest('tr'));
        dataObj.id = $(this).closest('tr')[0].id
        // Add your delete logic here
        console.log('Delete record with id:', dataObj.id);
        deleteRecord(dataObj);
    });
}

// Function to change the view based on selected type (upcoming or completed)
function changeView(selectedType) {
    csArray = selectedType === 'upcoming' ? upcomingCSArray : completedCSArray;
    var $viewSelector = $('#viewSelector');
    var $titleSpan = $viewSelector.find('.title');
    var $title = $(`#${selectedType}`).attr('aria-label');
    $viewSelector.attr('aria-label', $title);
    $titleSpan.text($title);
    $viewSelector.attr('title', $title);

    // Update table data based on selected type
    setCSTableData(undefined, "class-schedules");
}

// Function to handle AJAX requests and show processing messages
function appAjax(processingMsg, ajaxOptions) {
    notificationMsg.show(processingMsg);
    return webapi.safeAjax(ajaxOptions)
        .fail(function (response) {
            if (response.responseJSON) {
                alert("Error: " + response.responseJSON.error.message)
            } else {
                alert("Error: Web API is not available... ")
            }
        }).always(notificationMsg.hide);
}

// Definition of webAPIExampleTable with rendering, adding, updating, removing, and setting data functions
var webAPIExampleTable = (function () {
    // Template for table rows and overall table structure
    var trTpl = '<% _.forEach(data, function(data){ %>' +
        '<tr id="<%=data.id%>">' +
        '<% _.forEach(columns, function(col){ %>' +
        '<td style="min-width: 100px;" data-attribute="<%=col.name%>" data-label="<%=col.label%>" data-value="<%=data[col.name]%>">' +
        '<a href="<%= window.location.origin %>/class-schedules/<%=data.partialUrl%>/?refid=<%=data.id%>&accId=<%=accId%>"><%-data[col.name]%></a>' +
        '</td>' +
        '<% }) %>' +
        '<%=data.actions%>' +
        '</tr>' +
        '<% }) %>';
    var tableTpl = '<table class="table table-hover">' +
        '<thead>' +
        '<tr>' +
        '<% _.forEach(columns, function(col){ %>' +
        '<th data-column="<%=col.name%>" class="sortable"><%=col.label%><span class="sort-caret"></span></th>' +
        '<% }) %>' +
        '</tr>' +
        '</thead>' +
        '<tbody>' + trTpl + '</tbody>' +
        '</table>';

    // Return function with rendering and data manipulation methods
    return function (config) {
        var me = this,
            columns = config.columns,
            deleteHandler = config.deleteHandler,
            $table;

        // Render the table with specified columns and data
        me.render = function (el) {
            $table = $(el).html(_.template(tableTpl)({
                columns: columns,
                data: me.data,
                accId: accId
            })).find('table');
            bindTableEvents($table, {
                columns: columns,
                deleteHandler: deleteHandler,
                sortHandler: config.sortHandler
            });
        };

        // Add a new record to the table
        me.addRecord = function (record) {
            $table.find('tbody tr:first').before(_.template(trTpl)({
                columns: columns,
                data: [record],
                accId: accId
            }));
            bindRowEvents($table.find('tbody tr:first'), config);
        };

        // Update a record in the table
        me.updateRecord = function (attributeName, newValue, record) {
            $table.find('tr[data-id="' + record.id + '"] td[data-attribute="' + attributeName + '"]').text(newValue);
        };

        // Remove a record from the table
        me.removeRecord = function (record) {
            $table.find('tr[data-id="' + record.id + '"]').fadeTo("slow", 0.7, function () {
                $(this).remove();
            });
        };

        // Set data in the table and render it
        me.setData = function (data) {
            me.data = data;
            me.render($('#dataTable'));
        };
    };
})();

// Function to delete a record from the table
function deleteRecord(recordObj) {
    var response = confirm("Are you sure, you want to delete \"" + recordObj.name + "\" ?");
    if (response == true) {
        appAjax('Deleting...', {
            type: "DELETE",
            url: "/_api/contacts(" + recordObj.id + ")",
            contentType: "application/json",
            success: function (res) {
                table.removeRecord(recordObj);
            }
        });
    }
    return false;
}

// Function to sort the table based on a column
function sortTable(column) {
    var currentOrder = table.currentSortOrder || {};
    var sortDirection = currentOrder[column] === 'asc' ? 'desc' : 'asc';
    currentOrder[column] = sortDirection;
    table.currentSortOrder = currentOrder;
    var sortedData = _.sortBy(table.data, function (item) {
        if (column === "createdon@OData.Community.Display.V1.FormattedValue" || column === "bb_starttime@OData.Community.Display.V1.FormattedValue" || column === "bb_endtime@OData.Community.Display.V1.FormattedValue") {
            return new Date(item[column]);
        } else {
            return item[column];
        }
    });
    if (sortDirection === 'desc') {
        sortedData.reverse();
    }

    setCSTableData(sortedData, "class-schedules");

    $('th.sortable').each(function () {
        var $caret = $(this).find('.sort-caret');
        if ($(this).data('column') === column) {
            $caret.text(sortDirection === 'asc' ? '▲' : '▼');
        } else {
            $caret.text('');
        }
    });
}

// Function to filter the table based on a query
function filterTable(query) {
    var filteredData = csArray;
    if (query != null && query !== "") {
        filteredData = _.filter(csArray, function (record) {
            return _.some(record, function (value) {
                return value.toString().toLowerCase().includes(query.toLowerCase());
            });
        });
    }
    setCSTableData(filteredData, "class-schedules");
}

// Function to prepare and print the table content
function printTable() {
    $('#dataTable a').each(function () {
        var $this = $(this);
        $this.replaceWith($this.text());
    });
    var printContent = $('#dataTable').html();
    var originalContent = $('body').html();
    $('body').html(printContent);
    window.print();
    $('body').html(originalContent);
}

// Function to get the current date and time
function getCurrentDateTime() {
    var now = new Date();
    return now.toISOString();
}

// Function to extract data attributes from a table row element
function getDataObject(rowEl) {
    var $rowEl = $(rowEl),
        attrObj = {
            id: $rowEl.attr('data-id'),
            name: $rowEl.attr('data-name')
        };
    $rowEl.find('td').each(function (i, el) {
        var $el = $(el),
            key = $el.attr('data-attribute');
        if (key) {
            attrObj[key] = $el.attr('data-value');
        }
    });
    return attrObj;
}

// Function to bind events to table rows
function bindRowEvents(tr, config) {
    var $row = $(tr),
        dataObj = getDataObject($row);
    $.each(config.columns, function (i, col) {
        var $el = $row.find('td[data-attribute="' + col.name + '"]');
        $el.on('click', $.proxy(col.handler, $el, col, dataObj));
    });
}

// Function to bind events to the table
function bindTableEvents($table, config) {
    $table.find('tbody tr').each(function (i, tr) {
        bindRowEvents(tr, config);
    });
    $table.find('th.sortable').on('click', function () {
        var column = $(this).data('column');
        config.sortHandler(column);
    });
}